function [Density]=GetMaximumEntropySpectralEstimate(TrigonometricMoments,X)
%GetMaximumEntropySpectralEstimate This function evaluates a special density 
%function at the given location X (in radians) and returns the result as array of 
%shape identical to that of X. The density function produces the prescribed 
%trigonometric moments (starting with the zeroth moment) and has minimal Burg 
%entropy. If the given moments do not admit a positive density, it sets Density to 
%an empty array.
    nFullMoment=size(TrigonometricMoments,1);
    Toeplitz=toeplitz(TrigonometricMoments');
	Eigenvalues=eig(Toeplitz);
    if(all(Eigenvalues>0.0))
        EvaluationPolynomial=Toeplitz\eye(nFullMoment,1);
        Density=real(EvaluationPolynomial(1))./(2.0.*pi.*abs(polyval(conj(flipud(EvaluationPolynomial)),exp(1.0j.*X))).^2);
    else
		Density=[];
    end
end
