function [Phase,Weight]=GetPisarenkoEstimate(TrigonometricMoments)
%GetPisarenkoEstimate Given a vector of trigonometric moments without (!) a zeroth 
%moment this function computes a distribution consisting of 
%size(TrigonometricMoments,1) Dirac-delta distributions which describes this 
%measurement correctly.
%\return Phase holds phases in radians (zero to two pi) and Weight holds 
%        corresponding weights. Both arrays have the same length as the given 
%        vector. Each entry describes one Dirac-delta.
    % Construct the Toeplitz matrix and extract the eigenvector for the smallest 
    % eigenvalue
    Toeplitz=toeplitz([0.0;TrigonometricMoments]');
    [Eigenvectors,Eigenvalues]=eig(Toeplitz);
    [~,iMinEigenvalue]=min(diag(Eigenvalues));
    Polynomial=Eigenvectors(:,iMinEigenvalue);
    % Determine the locations of the Dirac-deltas
    DiracDeltaLocations=roots(fliplr(Polynomial'));
    % Compute the weights via a least squares system
    nMoment=size(TrigonometricMoments,1);
    Vandermonde=repmat(DiracDeltaLocations.',nMoment,1).^repmat((1:nMoment)',1,nMoment);
    Weight=real(Vandermonde\TrigonometricMoments);
    % Complete the output
    Angle=angle(DiracDeltaLocations);
    Phase=Angle+((Angle<0.0).*(2.0*pi));
end
