/*! \file
   This header defines functions for reconstructing distributions with minimal Burg 
   entropy matching given trigonometric moments. This is known as maximum entropy 
   spectral estimate.*/

#include "LevinsonAlgorithm.fx"

static const float PI=3.1415926535897932384626433832795f;

#define MOMENT_COUNT 1
/*! This function computes and returns the density at the given point of interest 
   (in radians) for the distribution with minimal Burg entropy matching the given 
   trigonometric moments. This is known as maximum entropy spectral estimate.
  \note The moments are not supposed to be normalized and the zeroth moment has to 
		be provided.*/
float GetMaximumEntropySpectralEstimate(float2 pTrigonometricMoment[MOMENT_COUNT+1],float Point){
	// Compute the polynomial, which extracts the constant part of other polynomials 
	// when it is used as left-hand-side in the dot product induced by the given 
	// trigonometric moments
	float2 EvaluationPolynomial[MOMENT_COUNT+1];
	SolveCanonicalToeplitzEquation(EvaluationPolynomial,pTrigonometricMoment);
	// Evaluate the probability density function
	float2 CirclePoint;
	sincos(Point,CirclePoint.y,CirclePoint.x);
	float2 PolynomialValue=EvaluatePolynomial(EvaluationPolynomial,Conjugate(CirclePoint));
	float ProbabilityDensity=EvaluationPolynomial[0].x*(0.5f/PI)/dot(PolynomialValue,PolynomialValue);
	return ProbabilityDensity;
}

#define MOMENT_COUNT 2
float GetMaximumEntropySpectralEstimate(float2 pTrigonometricMoment[MOMENT_COUNT+1],float Point){
	// Compute the polynomial, which extracts the constant part of other polynomials 
	// when it is used as left-hand-side in the dot product induced by the given 
	// trigonometric moments
	float2 EvaluationPolynomial[MOMENT_COUNT+1];
	SolveCanonicalToeplitzEquation(EvaluationPolynomial,pTrigonometricMoment);
	// Evaluate the probability density function
	float2 CirclePoint;
	sincos(Point,CirclePoint.y,CirclePoint.x);
	float2 PolynomialValue=EvaluatePolynomial(EvaluationPolynomial,Conjugate(CirclePoint));
	float ProbabilityDensity=EvaluationPolynomial[0].x*(0.5f/PI)/dot(PolynomialValue,PolynomialValue);
	return ProbabilityDensity;
}

#define MOMENT_COUNT 3
float GetMaximumEntropySpectralEstimate(float2 pTrigonometricMoment[MOMENT_COUNT+1],float Point){
	// Compute the polynomial, which extracts the constant part of other polynomials 
	// when it is used as left-hand-side in the dot product induced by the given 
	// trigonometric moments
	float2 EvaluationPolynomial[MOMENT_COUNT+1];
	SolveCanonicalToeplitzEquation(EvaluationPolynomial,pTrigonometricMoment);
	// Evaluate the probability density function
	float2 CirclePoint;
	sincos(Point,CirclePoint.y,CirclePoint.x);
	float2 PolynomialValue=EvaluatePolynomial(EvaluationPolynomial,Conjugate(CirclePoint));
	float ProbabilityDensity=EvaluationPolynomial[0].x*(0.5f/PI)/dot(PolynomialValue,PolynomialValue);
	return ProbabilityDensity;
}

#define MOMENT_COUNT 4
float GetMaximumEntropySpectralEstimate(float2 pTrigonometricMoment[MOMENT_COUNT+1],float Point){
	// Compute the polynomial, which extracts the constant part of other polynomials 
	// when it is used as left-hand-side in the dot product induced by the given 
	// trigonometric moments
	float2 EvaluationPolynomial[MOMENT_COUNT+1];
	SolveCanonicalToeplitzEquation(EvaluationPolynomial,pTrigonometricMoment);
	// Evaluate the probability density function
	float2 CirclePoint;
	sincos(Point,CirclePoint.y,CirclePoint.x);
	float2 PolynomialValue=EvaluatePolynomial(EvaluationPolynomial,Conjugate(CirclePoint));
	float ProbabilityDensity=EvaluationPolynomial[0].x*(0.5f/PI)/dot(PolynomialValue,PolynomialValue);
	return ProbabilityDensity;
}

#define MOMENT_COUNT 5
float GetMaximumEntropySpectralEstimate(float2 pTrigonometricMoment[MOMENT_COUNT+1],float Point){
	// Compute the polynomial, which extracts the constant part of other polynomials 
	// when it is used as left-hand-side in the dot product induced by the given 
	// trigonometric moments
	float2 EvaluationPolynomial[MOMENT_COUNT+1];
	SolveCanonicalToeplitzEquation(EvaluationPolynomial,pTrigonometricMoment);
	// Evaluate the probability density function
	float2 CirclePoint;
	sincos(Point,CirclePoint.y,CirclePoint.x);
	float2 PolynomialValue=EvaluatePolynomial(EvaluationPolynomial,Conjugate(CirclePoint));
	float ProbabilityDensity=EvaluationPolynomial[0].x*(0.5f/PI)/dot(PolynomialValue,PolynomialValue);
	return ProbabilityDensity;
}

#define MOMENT_COUNT 6
float GetMaximumEntropySpectralEstimate(float2 pTrigonometricMoment[MOMENT_COUNT+1],float Point){
	// Compute the polynomial, which extracts the constant part of other polynomials 
	// when it is used as left-hand-side in the dot product induced by the given 
	// trigonometric moments
	float2 EvaluationPolynomial[MOMENT_COUNT+1];
	SolveCanonicalToeplitzEquation(EvaluationPolynomial,pTrigonometricMoment);
	// Evaluate the probability density function
	float2 CirclePoint;
	sincos(Point,CirclePoint.y,CirclePoint.x);
	float2 PolynomialValue=EvaluatePolynomial(EvaluationPolynomial,Conjugate(CirclePoint));
	float ProbabilityDensity=EvaluationPolynomial[0].x*(0.5f/PI)/dot(PolynomialValue,PolynomialValue);
	return ProbabilityDensity;
}

#define MOMENT_COUNT 7
float GetMaximumEntropySpectralEstimate(float2 pTrigonometricMoment[MOMENT_COUNT+1],float Point){
	// Compute the polynomial, which extracts the constant part of other polynomials 
	// when it is used as left-hand-side in the dot product induced by the given 
	// trigonometric moments
	float2 EvaluationPolynomial[MOMENT_COUNT+1];
	SolveCanonicalToeplitzEquation(EvaluationPolynomial,pTrigonometricMoment);
	// Evaluate the probability density function
	float2 CirclePoint;
	sincos(Point,CirclePoint.y,CirclePoint.x);
	float2 PolynomialValue=EvaluatePolynomial(EvaluationPolynomial,Conjugate(CirclePoint));
	float ProbabilityDensity=EvaluationPolynomial[0].x*(0.5f/PI)/dot(PolynomialValue,PolynomialValue);
	return ProbabilityDensity;
}

#define MOMENT_COUNT 8
float GetMaximumEntropySpectralEstimate(float2 pTrigonometricMoment[MOMENT_COUNT+1],float Point){
	// Compute the polynomial, which extracts the constant part of other polynomials 
	// when it is used as left-hand-side in the dot product induced by the given 
	// trigonometric moments
	float2 EvaluationPolynomial[MOMENT_COUNT+1];
	SolveCanonicalToeplitzEquation(EvaluationPolynomial,pTrigonometricMoment);
	// Evaluate the probability density function
	float2 CirclePoint;
	sincos(Point,CirclePoint.y,CirclePoint.x);
	float2 PolynomialValue=EvaluatePolynomial(EvaluationPolynomial,Conjugate(CirclePoint));
	float ProbabilityDensity=EvaluationPolynomial[0].x*(0.5f/PI)/dot(PolynomialValue,PolynomialValue);
	return ProbabilityDensity;
}
