# -*- coding: utf-8 -*-
import numpy as np
from matplotlib import pyplot,colors

## \file
# Provides utility functions for plotting.

def GetSubplotLayout(nPlot):
    """Given a number of suplots to have in a figure this function returns an 
       adequate number of rows and columns of subplots as pair (nRow,nColumn)."""
    if(nPlot>0 and nPlot<12):
        nPlotRow=[1,1,1,2,2,2,2,2,3,3,3,3][nPlot-1];
    else:
        nPlotRow=max(1,int(nPlot**0.5));
    nPlotColumn=nPlot/nPlotRow if nPlot%nPlotRow==0 else (1+nPlot/nPlotRow);
    return (nPlotRow,nPlotColumn);

def LinearToSRGB(Linear):
    """Converts all scalars in the given array (which should be normalized to the 
       range from zero to one) from a linear scale to the sRGB scale."""
    return np.where(Linear<=0.0031308,12.92*Linear,1.055*Linear**(1.0/2.4)-0.055);

def ComplexToSRGB(ComplexValue):
    """Given an array of complex values this function constructs an array with an 
       additional last dimension of length 3. Along this dimension sRGB colors are 
       stored which represent the given complex values. The argument defines the 
       hue, magnitudes larger than one define the saturation and magnitudes smaller 
       than one define the value."""
    ComplexValue=np.nan_to_num(ComplexValue);
    HueSaturationValue=np.zeros(ComplexValue.shape+(3,),dtype=np.float);
    HueSaturationValue[...,0]=np.angle(ComplexValue)/(2.0*np.pi);
    HueSaturationValue[...,0]+=np.where(HueSaturationValue[:,:,0]<0.0,1.0,0.0);
    HueSaturationValue[...,1]=np.exp(-0.04*(1.0-np.maximum(np.abs(ComplexValue),1.0))**2);
    HueSaturationValue[...,2]=np.minimum(np.abs(ComplexValue),1.0);
    return LinearToSRGB(colors.hsv_to_rgb(HueSaturationValue));

def ShowComplexFunction(Function):
    """Plots an image of a complex-valued function on a two-dimensional domain using 
       a specialized colormap for complex functions. The argument defines the hue, 
       magnitudes larger than one define the saturation and magnitudes smaller than 
       one define the value."""
    pyplot.imshow(ComplexToSRGB(Function));
